#!/bin/sh
#######################################################################
# SYNTAX: tcm_restore.sh <backup_file> [custom_migration_script]
#
# PURPOSE: Restore a backup file containing the
# data files for a particular release, and add the
# configuration to this environment. If the backup
# is an older version, then migration will be
# done if the appropriate "migration functions" file
# is found.
#
#######################################################################

do_migration () {

   if [ ! -f /IBM/bin/tcm_restore_${from}_${to}.sh ]; then
      echo
      echo "Unsupported restore from $from to $to."
      echo
      exit 1
   else
      echo "Migrating data: $from -> $to"
      /IBM/bin/tcm_restore_${from}_${to}.sh
   fi
}


if [ $# -lt 1 ]; then
   echo "" &>2
   echo "Syntax: $0 <backup_file> [custom_migration_script]" &>2
   echo "" &>2

   exit 1
fi

BACKUP_FILE=$1
CUSTOM_MIGRATION_SCRIPT=$2

# Retrieve version information.
BACKUP_RELEASE=`tar -tzf $BACKUP_FILE | sed -n -e 1s/TCM_BACKUP\ //p | cut -d' ' -f1 | sed -e s/\"//g -e s/R//`
BACKUP_VERSION=`tar -tzf $BACKUP_FILE | sed -n -e 1s/TCM_BACKUP\ //p | cut -d' ' -f2 | sed -e s/\"//g -e s/SU//`
BACKUP_RELEASE=${BACKUP_RELEASE:=1}
BACKUP_VERSION=${BACKUP_VERSION:=2}
RESTORE_RELEASE=`grep linux_dist_rel /IBM/version | cut -d= -f2 | sed -e s/\"//g -e s/R//`
RESTORE_VERSION=`grep linux_dist_ver /IBM/version | cut -d= -f2 | sed -e s/\"//g -e s/SU//`



# Basic idea is to untar in /tmp/tcm/restore, then parse, mv, or rm 
# "known" files to thier new location. Any files left in /tmp/tcm/restore
# will be directly mv'd from /tmp/tcm/restore/... to /...
#
# Iteratively apply migration scripts so we can move from "a -> b -> c"
# without writing an "a -> c" migration script. No migration is needed
# for an "a -> a" restore.


# Setup temporary work environment.
rm -rf /tmp/tcm/restore
mkdir -p /tmp/tcm/restore
tar -xpzf $BACKUP_FILE --same-owner -C /tmp/tcm/restore >/dev/null 2>&1

# Export environment variables for migration scripts.
export BACKUP_VERSION
export RESTORE_VERSION

let rvar=$BACKUP_RELEASE
let vvar=$BACKUP_VERSION

let backup_value=$rvar*100+$vvar
let restore_value=$RESTORE_RELEASE*100+$RESTORE_VERSION

if [ $backup_value -gt $restore_value ]; then
   # This backup is from a *newer* release / version than this
   # restore image. This is not supported.

   echo
   echo "Error. Unable to restore data from a newer image ($BACKUP_RELEASE.$BACKUP_VERSION) to the current image ($RESTORE_RELEASE.$RESTORE_VERSION)."
   echo
   exit 2
fi

while [ $backup_value -lt $restore_value ]; do
   case $backup_value in
      102)
         from="$rvar.$vvar"
         let vvar=vvar+1
         let backup_value=$rvar*100+$vvar
         to="$rvar.$vvar"
 
         do_migration
         ;;
      103)
         from="$rvar.$vvar"
         let vvar=0
         let rvar=rvar+1
         let backup_value=$rvar*100+$vvar
         to="$rvar.$vvar"

         do_migration
         ;;
      200)
         from="$rvar.$vvar"
         let vvar=vvar+1
         let backup_value=$rvar*100+$vvar
         to="$rvar.$vvar"

         do_migration
         ;;
      *)
         echo "Error. Unknown backup (${rvar}-${vvar})"
         exit 1
         ;;
   esac
done

# Call custom migration script (if specified).
if [ -n "$CUSTOM_MIGRATION_SCRIPT" ]; then
   if [ -f $CUSTOM_MIGRATION_SCRIPT ]; then
      echo "Executing custom migration script: \"$CUSTOM_MIGRATION_SCRIPT\""
      $CUSTOM_MIGRATION_SCRIPT
   fi
fi

# Move remaining files.
echo "Restoring data."
chown -R KIOSK.KIOSK /tmp/tcm/restore/root/KIOSK
cp -Rfdpv /tmp/tcm/restore/* /

# Paranoid cleanup.
rm -rf /tmp/tcm/restore



